#####################################################################
# ***** BEGIN GPL LICENSE BLOCK *****
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
bl_info = {
    "name": "Camera Marks",
    "author": "teldredge",
    "version": (0, 0, 1),
    "blender": (2, 6, 1),
    "api": 42615,
    "location": "View3D > ToolShelf > Camera Marks",
    "description": "Camera Marks",
    "warning": "Beta",
    "wiki_url": "http://www.funkboxing.com",
    "tracker_url": "http://www.funkboxing.com",
    "category": "Camera"}
#####################################################################
#####################################################################
#####################################################################
"""
Very Simple UI to store and retreive camera loc/rot/dofdist/focal len
Stores properties in a text block
"""
#####################################################################
#####################################################################
#####################################################################

import bpy
import mathutils
#####################################################################
#####################################################################
#####################################################################
print('\n\n\n:-----GO-----')


###---CHECK IF TEXTBLOCK EXISTS, IF NOT, CREATE IT
try:
    markText = bpy.data.texts['CAMERAMARKS']
except:
    print('CREATING TEXT BLOCK')
    
    bpy.data.texts.new('CAMERAMARKS')
    markText = bpy.data.texts['CAMERAMARKS']    


thiscam = bpy.context.scene.camera

print(':cammarks textfile is ', markText)
print(':cammarks textfile has', len(markText.lines), 'lines')
print(':cammarks active cameras is ', thiscam)
print('-------------')

def vecToText(vc):
    tx = str(vc[0]) + "," + str(vc[1]) + "," + str(vc[2])
    return tx

def textToVec(tx):
    sl = tx.split(',')
    vc = mathutils.Vector((float(sl[0]), float(sl[1]), float(sl[2])))
    return vc

def listToString(list):
    str = ""
    for li in list:
        str = str + li + "|" 
    return str[0:len(str)-1]

def stringToList(str):
    #ll = []
    ll = str.split('|')
    return ll

def shortVecString(vc):
    vcstr = str(round(vc[0], 2)) + ', ' + str(round(vc[1], 2)) + ', ' + str(round(vc[2], 2))
    return vcstr

def camPropsStrList(cam):
    ###---generate string list of camera props
    cloc = vecToText(cam.location)
    crot = vecToText(cam.rotation_euler)
    cdof = str(cam.data.dof_distance)
    clen = str(cam.data.lens)
    ll = [cloc, crot, cdof, clen]
    sl = listToString(ll)
    return sl

def writeCamProps(namestr, propstr, text):
    text.write(namestr + '|' + propstr + '\n')

def setCamProps(cam, props):
    cam.location = textToVec(props[1])
    cam.rotation_euler = textToVec(props[2])
    cam.data.dof_distance = float(props[3])
    cam.data.lens = float(props[4])

def readCamProps(marknum, text):
    lstr = text.lines[marknum].body
    li = stringToList(lstr)
    return li

print('-------------')


def deleteMarkerLine(marknum, text):
    ntext = ''
    print('NUMBER OF MARKS=', len(text.lines))
    for lnum in range(len(text.lines)-1):
        print('checking:', lnum, marknum)
        if lnum != marknum:
            print('adding', lnum)
            ntext = ntext + text.lines[lnum].body + '\n'
            #ntext = ntext + text.lines[lnum].body
    print('NEW TEXT TO WRITE:', ntext)
    #text.clear()
    #text.write(ntext[0:len(ntext)-2])
    #text.write(ntext)
    text.from_string(ntext)
    print('NUMBER OF MARKS=', len(text.lines))

        
    




#####################################################################
################################ UI #################################
#####################################################################
bpy.types.Scene.CAMMARK = bpy.props.IntProperty(
    name = "cammark", description = "camera mark")
bpy.types.Scene.CAMMARK = 0
bpy.types.Scene.markname = bpy.props.StringProperty(description = "Name of mark")

####----TESTING
#writeCamProps('markName', camPropsStrList(thiscam), markText)
#writeCamProps('markName', camPropsStrList(thiscam), markText)
#writeCamProps('markName', camPropsStrList(thiscam), markText)


#deleteMarkerLine(bpy.types.Scene.CAMMARK, markText)
#deleteMarkerLine(1, markText)



class addCameraMarkOperator(bpy.types.Operator):
    '''DO SOMETHING WITH THIS OPERATOR!!!'''
    bl_idname = "object.addcameramark_operator"
    bl_label = "add Camera Mark Operator"
    def execute(self, context):
        writeCamProps('markName', camPropsStrList(thiscam), markText)
        bpy.types.Scene.CAMMARK += 1
        print(':ADDED MARK', 'markName', camPropsStrList(thiscam), markText)
        return {'FINISHED'}

class delCameraMarkOperator(bpy.types.Operator):
    '''DO SOMETHING WITH THIS OPERATOR!!!'''
    bl_idname = "object.delcameramark_operator"
    bl_label = "del Camera Mark Operator"
    def execute(self, context):
        if len(markText.lines) > 2:
            deleteMarkerLine(bpy.types.Scene.CAMMARK, markText)
            #writeCamProps('somenewName', camPropsStrList(thiscam), camtext)
            #print('ADDED SHOT', 'somenewName', camPropsStrList(thiscam), camtext)
            tempint = bpy.types.Scene.CAMMARK - 1
            markCount = len(markText.lines)-1
            bpy.types.Scene.CAMMARK = tempint % markCount
            markprops = readCamProps(bpy.types.Scene.CAMMARK, markText)
        return {'FINISHED'}

class nextCameraMarkOperator(bpy.types.Operator):
    '''DO SOMETHING WITH THIS OPERATOR!!!'''
    bl_idname = "object.nextcameramark_operator"
    bl_label = "next Camera Mark Operator"
    def execute(self, context):
        tempint = bpy.types.Scene.CAMMARK + 1
        markCount = len(markText.lines)-1
        bpy.types.Scene.CAMMARK = tempint % markCount
        markprops = readCamProps(bpy.types.Scene.CAMMARK, markText)
        print(':setting camera mark to next number: ',  bpy.types.Scene.CAMMARK)
        setCamProps(thiscam, markprops)
        return {'FINISHED'}

class lastCameraMarkOperator(bpy.types.Operator):
    '''DO SOMETHING WITH THIS OPERATOR!!!'''
    bl_idname = "object.lastcameramark_operator"
    bl_label = "last Camera Mark Operator"
    def execute(self, context):
        tempint = bpy.types.Scene.CAMMARK - 1
        markCount = len(markText.lines)-1
        bpy.types.Scene.CAMMARK = tempint % markCount
        markprops = readCamProps(bpy.types.Scene.CAMMARK, markText)
        print(':setting camera mark to last number: ',  bpy.types.Scene.CAMMARK)
        setCamProps(thiscam, markprops)
        return {'FINISHED'}


class OBJECT_PT_dummy(bpy.types.Panel):
    bl_label = "Camera Marks"
    bl_space_type = "VIEW_3D"
    bl_region_type = "TOOLS"
    bl_context = "objectmode"

    def draw(self, context):
        scn = context.scene
        layout = self.layout
        col = layout.column()        

        col.label('Current Mark: ' + str(bpy.types.Scene.CAMMARK+1) + ' of ' + str(len(markText.lines)-1))
        #col.prop(scn, 'CAMMARK')
        col.label('Loc: ' + shortVecString(thiscam.location))
        col.label('Roc: ' + shortVecString(thiscam.rotation_euler))
        col.label('Lens: ' + str(round(thiscam.data.lens,2)))        
        col.operator('object.addcameramark_operator', text = 'addmark')
        col.operator('object.nextcameramark_operator', text = 'nextmark')
        col.operator('object.lastcameramark_operator', text = 'lastmark')
        col.operator('object.delcameramark_operator', text = 'delmark')
        #col.label('shotname')
        #col.prop_search(scn, "shotname",  context.scene, "objects")        
 

        
def register():
    bpy.utils.register_class(addCameraMarkOperator)
    bpy.utils.register_class(nextCameraMarkOperator)
    bpy.utils.register_class(lastCameraMarkOperator)
    bpy.utils.register_class(delCameraMarkOperator)
    bpy.utils.register_class(OBJECT_PT_dummy)

def unregister():
    bpy.utils.unregister_class(dummyOperator)    
    bpy.utils.unregister_class(OBJECT_PT_dummy)

if __name__ == "__main__":
    #print(len(markText.lines))
    if len(markText.lines) < 2:
        writeCamProps('markName', camPropsStrList(thiscam), markText) 
    register()
    pass

